--  The package Sockets.Unix has not been tested at all.
--  This dummy test program just forces compilation, to check for
--  syntax errors.

with Sockets.Unix;
procedure Text_Unix_Sockets is
begin
   null;
end Test_Unix_Sockets;
